#ifndef GUARD_TagFactory_h
#define GUARD_TagFactory_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file TagFactory.h
/// @brief Factory class to produce tag processor object.

#include <map>
#include <string>

#include <exception>

#include "tag.h"

#include "fixedallocator.h"

#include "misc.h"

/// @brief Factory class to produe tag processor object.
class TagFactory : public SmallObject
{
public:
    static TagFactory& Instance()
    {
        static TagFactory instance;

        return instance;
    }

    /// @brief Register id to callback map.
    /// @param id Identification string.
    /// @param creator Callback function to produce object.
    /// @return Success of register.
    bool Register(const char* const id, std::auto_ptr<Tag> (*creator)(const std::string&))
    {
        return callbacks_.insert( callback::value_type(id, creator) ).second;
    }

    /// @brief Unregister id from callback map.
    /// @param id Identification string.
    /// @return Success of unregister.
    bool Unregister(const std::string& id)
    {
        return callbacks_.erase(id) == 1;
    }

    /// @brief Check factory has filename's type.
    /// @param filename Filename to check.
    /// @return Return true if factory has filename's type.
    bool Exist(const std::string& filename)
    {
        callback::const_iterator i = callbacks_.find(GetExt(filename));
	    
        if(i != callbacks_.end())
            return true;
        else
            return false;	
    }
	
    /// @brief Create tag processor object.
    /// @param filename filename to process.
    /// @return Tag processor object.
    std::auto_ptr<Tag> Create(const std::string& filename)
    {
        callback::const_iterator i = callbacks_.find(GetExt(filename));

        if(i != callbacks_.end()) {

            try {

                return (i->second)(filename);

            } catch(const Exception::SaguaException& ex) {

                return (callbacks_["empty"])(filename);
            }

        } else
            return (callbacks_["empty"])(filename);
    }

    /// @brief True if there is nothing registered.
    bool Empty() const
    {
        return callbacks_.empty();
    }

private:
    TagFactory();

    typedef std::map<std::string, std::auto_ptr<Tag> (*)(const std::string&)> callback;
    callback callbacks_;	    ///< Callback map.
};

#endif
