#ifndef GUARD_audio_h
#define GUARD_audio_h
/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file audio.h
/// @brief Class for libao wrapping.

#include <map>
#include <string>

#include <boost/bind.hpp>
#include <boost/lambda/lambda.hpp>

#include <ao/ao.h>

/// @brief Namespace for libao wrapper class.
namespace Libao {

    typedef std::pair<const std::string, std::string> OptionPair;
    typedef std::map<std::string, std::string> OptionMap;
    typedef OptionMap::const_iterator OptionMapCIter;

    /// @brief Wrapper class for ao_sample_format.
    class Format
    {
        friend class Device;

    public:
        bool Set(unsigned int bits, unsigned int channels, unsigned int sampleRate);

        unsigned int GetBits() const
        {
            return format_.bits;
        }

        unsigned int GetChannels() const
        {
            return format_.channels;
        }

        unsigned int GetSampleRate() const
        {
            return format_.rate;
        }

    private:
        ao_sample_format format_;
    };

    /// @brief Wrapper class for ao_option.
    class Option
    {
        friend class Device;

    public:
        Option()
            : option_(0)
        {}

        ~Option()
        {
            Free();
        }

        void Free()
        {
            ao_free_options(option_);

            option_ = 0;
        }

        void Append(const OptionMap& option);

    private:
        Option(const Option&);
        Option operator=(const Option&);

        ao_option* option_;

        void AppendPair(const OptionPair& pair);
    };

    /// @brief Wrapper class for ao_device.
    class Device
    {
    public:
        Device()
            : device_(0)
        {}

        ~Device()
        {
            Close();
        }

        bool Open(int id, Format& format)
        {
            device_ = ao_open_live(id, &format.format_, 0);

            return Check();
        }

        bool Open(int id, Format& format, Option& option)
        {
            device_ = ao_open_live(id, &format.format_, option.option_);

            return Check();
        }

        bool Check() const
        {
            return 0 != device_;
        }

        void Close()
        {
            if(Check()) {
                
                ao_close(device_);
                device_ = 0;
            }
        }

        int Play(char* buffer, unsigned int size)
        {
            return ao_play(device_, buffer, size);
        }

    private:
        Device(const Device&);
        Device operator=(const Device&);

        ao_device* device_;
    };

    /// @brief Wrapper class for library setup and provide interface.
    class AudioDriver
    {
    public:
        /// @brief Destructor of Audio.
        ///
        /// NOTES. After call ao_shutdown(), you cannot initialize libao again probably.
        /// This is a one of reason why this class is singleton.
        ~AudioDriver()
        {
            ao_shutdown();
        }

        /// @brief Create Audio Object.
        static AudioDriver& Instance()
        {
            static AudioDriver instance_;

            return instance_;
        }

        void SetDriver(const char* audioDriver, const OptionMap& option);

        bool OpenDevice(unsigned int bits, unsigned int channels, unsigned int sampleRate);

        void Close();

        bool Check() const;

        void Play(char* buffer, unsigned int size);

    private:
        AudioDriver()
            : driverId_(-1)
        {
            ao_initialize();
        }

        AudioDriver(const AudioDriver&);
        AudioDriver& operator=(const AudioDriver&);

        int driverId_;

        Format format_;
        Option option_;
        Device device_;

        void GetDriverId(const char* driver);
    };

    void SetDefault(Format& format);

    bool TestDriver(const char* const driverName);
    bool TestDriver(const char* const driverName, const OptionMap& option);

    bool TestDriverId(int id);
    bool TestDriverId(int id, const OptionMap& option);

    bool IsDefault(const char* driver);
}

inline void Libao::Option::Append(const OptionMap& option)
{
    std::for_each(option.begin(), option.end(), boost::bind(&Libao::Option::AppendPair, this, _1));
}

inline void Libao::Option::AppendPair(const OptionPair& pair)
{
    ao_append_option(&option_, pair.first.c_str(), pair.second.c_str());
}

inline void Libao::AudioDriver::Close()
{
    device_.Close();
}

inline bool Libao::AudioDriver::Check() const
{
    return device_.Check();
}

inline void Libao::AudioDriver::Play(char* buffer, unsigned int size)
{
    device_.Play(buffer, size);
}

inline void Libao::SetDefault(Format& format)
{
    format.Set(16, 2, 44100);
}

inline bool Libao::IsDefault(const char* driver)
{
    return driver[0] == 'd';
}

#endif
