/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file main.cc
/// @brief main function file

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdio>
#include <ctime>

#include <vector>
#include <string>

#include <memory>

#include <unistd.h>

#include <gtkmm/main.h>
#include <sigc++/sigc++.h>

#include <libintl.h>

#include <boost/program_options.hpp>

#include "../include/mainmanager.h"
#include "../include/sockethandler.h"
#include "../include/exception.h"
#include "../include/trace.h"

/** @mainpage Main Page
    @section Intro
    Sagua is a music player for Gtk+/Gnome.

    If you want to look around pages, i recommend you to see @ref MainManager.
    @section Developer
    HG, Rhee <ez8tank@krpost.net>
    @section History
    - 2005-05-29 v0.2.5 released
    - 2005-05-23 v0.2.4 released
    - 2005-04-16 v0.2.3 released
    - 2005-03-03 v0.2.2 released
    - 2005-02-22 v0.2.1 released
    - 2005-02-12 v0.2.0 released
    - 2005-01-27 v0.1.2 released
    - 2005-01-20 v0.1.1 released
    - 2005-01-13 v0.1.0 released
    @section Requirement
    - Gtkmm 2.4 over
    - libao
    - libmad
    - libogg
    - libvorbis

    After v0.2.0
    - id3lib
	
    else
	    
    - libid3tag

    - Optional
    -# faad2
    -# ACE library
*/

namespace SaguaGlobals {

    MainManager* mainManager = 0;

    PlayControlDisplay& playControlDisplay = *mainManager;
    PlayerInterface& playerInterface = *mainManager;
    SettingInterface& settingInterface = *mainManager;

}

void ParseCommandOptions(int argc, char* argv[], std::vector<std::string>& inputFiles)
{
    try {

        boost::program_options::options_description generic("Generic options");
        generic.add_options()
            ("help,h", "produce help message")
            ("version,v", "display version")
#ifdef ENABLE_DEBUGTRACE
            ("debug,d", "display debug message")
#endif
            ;
    
        boost::program_options::options_description hidden("Hidden options");
        hidden.add_options()
            ("input-file",
             boost::program_options::value< std::vector<std::string> >(&inputFiles),
             "input file")
            ;

        boost::program_options::options_description cmdline_options;
        cmdline_options.add(generic).add(hidden);

        boost::program_options::options_description visible("Allowed options");
        visible.add(generic);
        
        boost::program_options::positional_options_description p;
        p.add("input-file", -1);
        
        boost::program_options::variables_map vm;
        store(boost::program_options::command_line_parser(argc, argv).
              options(cmdline_options).positional(p).run(), vm);

        notify(vm);

        if(vm.count("help")) {

            std::printf("Usage : %s [options] FILES\n", argv[0]);
            std::cout << visible << '\n';

            exit(0);
        }

        if(vm.count("version")) {

            std::cout << PACKAGE_NAME << ' ' << VERSION << '\n';

            exit(0);
        }

    } catch(std::exception& ex) {

        std::printf("%s\n", ex.what());

        exit(1);
    }
}

std::auto_ptr<UnixSocketHandler> CreateSocketHandler()
{
    try {

        std::auto_ptr<UnixSocketHandler> rtn(new UnixSocketHandler);

        return rtn;

    } catch(const Exception::SaguaException& ex) {

        std::printf("%s\n", ex.what());

        exit(0);
    }
}

/// @brief main function
///
/// Process command line arguments. If socket is connected to already executed
/// program, send arguments to that and exit. 
/// Or execute little server thread. After that, run MainManager loop.
int main(int argc, char* argv[])
{
    std::vector<std::string> inputFiles;

    ParseCommandOptions(argc, argv, inputFiles);

    DEBUGTRACE("main");

    srand( time(NULL) );
    
    if(!Glib::thread_supported()) {

        DEBUGTRACE_MSG("thread init");

        Glib::thread_init();
    }

    std::auto_ptr<UnixSocketHandler> socketHandler(CreateSocketHandler());

    if(socketHandler->Connect() && !inputFiles.empty()) {

        socketHandler->Send(inputFiles);
	
        exit(0);
	
    } else 
        socketHandler->StartServerLoop();

    Gtk::Main kit(argc, argv);

    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);

    GUIMainManager& guiMainManager = MainManagerImpl::Instance();

    socketHandler->ConnectSlot(sigc::mem_fun(guiMainManager, &GUIMainManager::SocketReceived));
    guiMainManager.SignalReceived().connect(sigc::mem_fun(*socketHandler, &UnixSocketHandler::ReceivedData));

    if(!inputFiles.empty())
        guiMainManager.PushList(inputFiles);

    SaguaGlobals::mainManager = &guiMainManager;
    SaguaGlobals::mainManager->Run();

    return 0;
}
