/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file mainmanager-c.cc
/// @brief Class for main program control in console.

#include <cstdio>
#include <cctype>

#include <glibmm.h>

#include "../include/mainmanager-c.h"

#include "../include/audio.h"
#include "../include/mixer.h"

#include "../include/playlist-c.h"

#include "../include/misc.h"

#include "../include/exception.h"

#include "audiobuffer.cc"
template class Audio::AudioBuffer<Libao::AudioDriver>;

#include "playcontroller.cc"
template class PlayControllerImpl<SaguaAudioBuffer>;

namespace SaguaGlobals {

    extern std::auto_ptr<SaguaPreference> preference;
}

/// @brief Constructor.
/// @throw SaguaException Exception on construct.
CUIMainManager::CUIMainManager()
    : isCharSetUTF8_(Glib::get_charset()),
      audio_(Libao::AudioDriver::Instance()),
      audioBuffer_(new SaguaAudioBuffer(*SaguaGlobals::preference, audio_)),
      mixer_(new Audio::OSSMixer(SaguaGlobals::preference->GetString("MixerDev"))),
      playController_(new PlayControllerImpl<SaguaAudioBuffer>(*audioBuffer_, *this, *this, *this)),
      playlist_(new CUIPlaylistImpl),
      mainLoop_(Glib::MainLoop::create())
{
    SetTermAttr();

    SetDriver();

    audioBuffer_->ConnectSignalTimerUpdate(sigc::mem_fun(*this, &MainManager::TimeUpdate));

    //playController_->SignalSetDriver().connect(sigc::mem_fun(*this, &MainManager::SetDriver));

    playController_->SetItemSelector(*playlist_, ItemSelectorCreator::NORMAL);

#ifdef ENABLE_HTTP
    //playlist_->SignalConvertUrlEncoding().connect(sigc::mem_fun(*this, &SettingInterface::GetConvertUrlToCurrentEncoding));
#endif

    if(!SaguaGlobals::preference->SuccessLoadConfig()) {

        //SavePreference();

        //RunPrefWnd();
    }

    Glib::signal_io().connect(sigc::mem_fun(*this, &CUIMainManager::ReadKey), 0, Glib::IO_IN);
}

CUIMainManager::~CUIMainManager()
{
    RestorePrevTermAttr();
}

/// @brief Run main loop;
void CUIMainManager::Run()
{
    mainLoop_->run();
}

SaguaPreference& CUIMainManager::GetPreference()
{
    return *SaguaGlobals::preference;
}

/// @todo Remove this.
LastState& CUIMainManager::GetLastState()
{
    return lastState_;
}

void CUIMainManager::DoSavePreference()
{
    ProgramFile& programFile(ProgramFile::Instance());

    if(programFile.CheckDir()) {
	
        std::ofstream prefFile(programFile.GetConfig());

        if(prefFile)
            SaguaGlobals::preference->Save(prefFile);
    }
}

#ifdef ENABLE_HTTP
/// @brief Convert url to current encoding or not.
bool CUIMainManager::GetConvertUrlToCurrentEncoding() const
{
    return !isCharSetUTF8_ && SaguaGlobals::preference->GetBool("GeneralConvertUrlToCurrentEncoding");
}
#endif

/// @brief Device is opened or not.
/// @sa Libao::Audio::CheckOpenDevice
bool CUIMainManager::CheckOpenDevice()
{
    return audio_.Check();
}

/// @brief Unpause audio buffer play.
///
/// Unpause audio buffer play and skip play buffered sample.
/// @sa Audio::PauseToggle
void CUIMainManager::DoUnpause()
{
    audioBuffer_->Unpause();
}

bool CUIMainManager::ReadKey(Glib::IOCondition io)
{
    if(io & Glib::IO_IN) {

        char key = 0;
        if(read(0, &key, 1) > 0) {

            switch(std::toupper(key)) {
            case PLAY_PREV:
                playController_->Prev();
                break;

            case PLAY_NEXT:
                playController_->Next();
                break;

            case PLAY_PAUSE:
                audioBuffer_->PauseToggle(false);
                break;

            case PLAY_STOP:
            case QUIT:
                mainLoop_->quit();
                break;

            case VOL_INCREASE:
                VolumnIncrease();
                break;

            case VOL_DECREASE:
                VolumnDecrease();
                break;
            }
        }
    }

    return true;
}

inline void CUIMainManager::PrintVolumn()
{
    std::fprintf(stderr, "\rVolumn : (%d/100)", static_cast<int>(GetVolumn()));
}

void CUIMainManager::RestorePrevTermAttr()
{
    tcsetattr(0, TCSAFLUSH, &origTermios_);
}

/// @throw TermException Exception on get/set term attr.
void CUIMainManager::SetTermAttr()
{
    termios thisTermios;

    if(tcgetattr(0, &thisTermios) < 0)
        throw Exception::TermException("Cannot get term attr");

    origTermios_ = thisTermios;

    thisTermios.c_lflag &= ~(ICANON|ECHO);
    thisTermios.c_cc[VMIN] = 1;
    thisTermios.c_cc[VTIME] = 0;

    if(tcsetattr(0, TCSANOW, &thisTermios) < 0)
        throw Exception::TermException("Cannot set term attr");
}

/// @brief Get current play time from audio output.
long int CUIMainManager::GetCurrentTime()
{
    return audioBuffer_->GetCurrentTime();
}

/// @brief Refresh Mixer widget.
/// @sa MainWnd::RefreshMixer
/// @todo Remove this.
void CUIMainManager::Refresh()
{
    mixer_->SetDevice(SaguaGlobals::preference->GetString("MixerDev"));
}

/// @brief Get current PCM volumn.
/// @sa Mixer::GetVolumn
double CUIMainManager::GetVolumn() const
{
    return mixer_->GetVolumn();
}

/// @brief Set PCM volumn.
/// @param volumn Value to set. 0 ~ 100.
/// @sa Mixer::SetVolumn
void CUIMainManager::SetVolumn(double volumn)
{
    mixer_->SetVolumn(volumn);
}

/// @brief Mixer Volumn increase.
/// @sa MainWnd::VolumnIncrease 
void CUIMainManager::VolumnIncrease()
{
    Audio::IncreaseVolumn(*mixer_);

    PrintVolumn();
}

/// @brief Mixer Volumn decrease.
/// @sa MainWnd::VolumnDecrease
void CUIMainManager::VolumnDecrease()
{
    Audio::DecreaseVolumn(*mixer_);

    PrintVolumn();
}

bool CUIMainManager::InPlay()
{
    return playController_->InPlay();
}

/// @brief Callback function used when play is finished.
/// @sa MainWnd::FinishPlay
void CUIMainManager::Finished()
{
    std::printf("\n");

    playController_->Finish();
}

/// @brief Callback function used when player need to update timer.
/// @sa MainWnd::TimeUpdate
void CUIMainManager::TimeUpdate()
{
    std::fprintf(stderr,
                 "\r(%s/%s)",
                 ConvertSecondToTimeString(audioBuffer_->GetCurrentTime()).c_str(),
                 playlist_->CurrentItemTime().c_str());
}

void CUIMainManager::DoUpdateItemRead()
{

}

void CUIMainManager::DoUpdatePlay()
{
    std::printf("Title   : %s\n", playlist_->CurrentItemTitle().c_str());

    // when some verbose option
    if(1)
        FileInfo();
}

void CUIMainManager::DoUpdateStop()
{
    mainLoop_->quit();
}

void CUIMainManager::DoUpdateItemRemoved()
{

}

/// @brief Show current file information.
/// @sa MainWnd::CurrentFileInfo
void CUIMainManager::FileInfo()
{
    playlist_->ShowCurrentFileInfo();
}

void CUIMainManager::PushList(const std::vector<std::string>& list)
{
    playlist_->Add(list);

    playController_->Play();
}

/// @brief Set MainWnd statusbar text and update it.
/// @sa MainWnd::SetStatusbar
void CUIMainManager::SetStatusbar(const Glib::ustring& text)
{
    std::printf("%s\n", text.c_str());
}

/// @brief Set MainWnd statusbar text.
/// @sa MainWnd::SetStatusbarText
void CUIMainManager::SetStatusbarText(const Glib::ustring& text)
{
    std::printf("%s\n", text.c_str());
}

/// @brief Update MainWnd statusbar to stored text.
/// @sa MainWnd::UpdateStatusbar
void CUIMainManager::UpdateStatusbar()
{}

/// @brief Update MainWnd statusbar to stored text and clear at timeout.
/// @sa MainWnd::UpdateStatusbarTimeoutClear
void CUIMainManager::UpdateStatusbarTimeoutClear()
{}

/// @brief Clear MainWnd statusbar.
/// @sa MainWnd::ClearStatusbar
void CUIMainManager::ClearStatusbar()
{}

/// @brief Clear MainWnd statusbar at timeout.
/// @sa MainWnd::TimeoutClearStatusbar
void CUIMainManager::TimeoutClearStatusbar()
{}

void CUIMainManager::DoSetDriver()
{
    std::map<std::string, std::string> option;
    SaguaGlobals::preference->GetAudioDriverOption(option);

    audio_.SetDriver(SaguaGlobals::preference->GetString("AudioDriver").c_str(), option);
}

/// @brief Open Audio device.
/// @param bits Bits per sample.
/// @param channels Number of channels.
/// @param sampleRate Sample rate per sample.
/// @return Success of open.
/// @sa Libao::Audio::OpenDevice
bool CUIMainManager::OpenDevice(unsigned int bits, unsigned int channels, unsigned int sampleRate)
{
    if(audio_.OpenDevice(bits, channels, sampleRate)) {

        try {

            audioBuffer_->Start(bits, channels, sampleRate);

        } catch(const Glib::ThreadError& ex) {

            // exception on buffer thread

            return false;
        }

        return true;

    } else {

        //Stop();

        //signalAudioDeviceOpenFailed_();

        std::printf("Cannot open audio device\n");

        mainLoop_->quit();

        return false;
    }
}
