/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file sagua-c.cc
/// @brief main function for console.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdio>
#include <cstdlib>
#include <ctime>

#include <string>
#include <vector>
#include <memory>

#include <glibmm.h>

#include <libintl.h>

#include <boost/program_options.hpp>

#include "../include/preference.h"
#include "../include/sockethandler.h"
#include "../include/exception.h"
#include "../include/trace.h"

#include "../include/mainmanager-c.h"

namespace SaguaGlobals {

    MainManager* mainManager = 0;

    PlayControlDisplay& playControlDisplay = *mainManager;
    PlayerInterface& playerInterface = *mainManager;
    SettingInterface& settingInterface = *mainManager;

    std::auto_ptr<SaguaPreference> preference;
}

void ParseCommandOptions(int argc, char* argv[], std::vector<std::string>& inputFiles)
{
    try {

        boost::program_options::options_description generic("Generic options");
        generic.add_options()
            ("help,h", "produce help message")
            ("version,v", "display version")
            ("disable-load,l", "Disable loading saved configuration")
#ifdef ENABLE_DEBUGTRACE
            ("debug,d", "display debug message")
#endif
            ;
    
        boost::program_options::options_description config("Configuration");
        config.add_options()
            ("driver,a", boost::program_options::value<std::string>(),
             "set audio driver")
            ("driveroption", boost::program_options::value<std::string>(),
             "set audio driver option")
            ("mixer,m", boost::program_options::value<std::string>(),
             "set mixer device")
#ifdef ENABLE_HTTP
            ("url", "convert url to current encoding")
#endif
            ;

        boost::program_options::options_description playMode("Play mode");
        playMode.add_options()
            ("repeat,r", "Repeat")
            ("random,z", "Random")
            ;

        boost::program_options::options_description effect("Effector");
        effect.add_options()
            ("bb", "Bass boost")
            ("bb2", "Bass boost2")
            ("tb", "High frequency boost")

            ("se", "Stereo enhance")
            ("sev",
             boost::program_options::value<double>(),
             "Stereo enhance value (0.1 ~ 2.0)")

            ("rm", "Room - space effector")
            ("rmv",
             boost::program_options::value<double>(),
             "Room effector value (0.1 ~ 1.0)")

            ("rm2", "Room2 - space effector")
            ("rm2v",
            boost::program_options::value<double>(),
            "Room2 effector value (0.1 ~ 1.0)")

            ("hm", "Harmony")
            ("hmv",
             boost::program_options::value<double>(),
             "Harmony effector value (0.1 ~ 1.0)")

            ("rh", "Rhythme")
            ("rhv",
             boost::program_options::value<double>(),
             "Rhythme effector value (0.1 ~ 1.5)")

            ("ds", "Distortion")
            ("dsv",
             boost::program_options::value<double>(),
             "Distortion effector value (1.0 ~ 10.0)")
            ;

        boost::program_options::options_description eq("Equalizer");
        eq.add_options()
            ("eq", "Use equalizer")
            ("eq50",
             boost::program_options::value<float>(),
             "50 Hz value (-12.0 ~ 12.0)")
            ("eq100",
             boost::program_options::value<float>(),
             "100 Hz value (-12.0 ~ 12.0)")
            ("eq200",
             boost::program_options::value<float>(),
             "200 Hz value (-12.0 ~ 12.0)")
            ("eq400",
             boost::program_options::value<float>(),
             "400 Hz value (-12.0 ~ 12.0)")
            ("eq800",
             boost::program_options::value<float>(),
             "800 Hz value (-12.0 ~ 12.0)")
            ("eq1500",
             boost::program_options::value<float>(),
             "1500 Hz value (-12.0 ~ 12.0)")
            ("eq3k",
             boost::program_options::value<float>(),
             "3 kHz value (-12.0 ~ 12.0)")
            ("eq5k",
             boost::program_options::value<float>(),
             "5 kHz value (-12.0 ~ 12.0)")
            ("eq7k",
             boost::program_options::value<float>(),
             "7 kHz value (-12.0 ~ 12.0)")
            ("eq10k",
             boost::program_options::value<float>(),
             "10 kHz value (-12.0 ~ 12.0)")
            ("eq15k",
             boost::program_options::value<float>(),
             "15 kHz value (-12.0 ~ 12.0)")
            ;

        boost::program_options::options_description hidden("Hidden options");
        hidden.add_options()
            ("input-file",
             boost::program_options::value< std::vector<std::string> >(&inputFiles),
             "input file")
            ;

        boost::program_options::options_description cmdline_options;
        cmdline_options.add(generic).add(config).add(playMode).add(effect).add(eq).add(hidden);

        boost::program_options::options_description visible("Allowed options");
        visible.add(generic).add(config).add(playMode).add(effect).add(eq);
        
        boost::program_options::positional_options_description p;
        p.add("input-file", -1);
        
        boost::program_options::variables_map vm;
        store(boost::program_options::command_line_parser(argc, argv).
              options(cmdline_options).positional(p).run(), vm);

        notify(vm);

        if(vm.count("help")) {

            std::printf("Usage : %s [options] FILES\n", argv[0]);
            std::cout << visible << '\n';

            exit(0);
        }

        if(vm.count("version")) {

            std::cout << PACKAGE_NAME << ' ' << VERSION << '\n';

            exit(0);
        }

        SaguaGlobals::preference = 
            std::auto_ptr<SaguaPreference>(new SaguaPreference(!vm.count("disable-load")));

        if(vm.count("bb"))
            SaguaGlobals::preference->Set("EfxBassBoost", true);

        if(vm.count("bb2"))
            SaguaGlobals::preference->Set("EfxBassBoost2", true);

        if(vm.count("tb"))
            SaguaGlobals::preference->Set("EfxTrebleBoost", true);

        if(vm.count("se"))
            SaguaGlobals::preference->Set("EfxStereoEnhance", true);

        if(vm.count("sev"))
            SaguaGlobals::preference->Set("EfxStereoEnhanceAmount", vm["sev"].as<double>());

        SaguaGlobals::preference->UpdateEffectUse();

        if(!vm.count("input-file")) {

            std::printf("No input files\n");

            exit(0);
        }

    } catch(std::exception& ex) {

        std::printf("%s\n", ex.what());

        exit(1);
    }
}

std::auto_ptr<UnixSocketHandler> CreateSocketHandler()
{
    try {

        std::auto_ptr<UnixSocketHandler> rtn(new UnixSocketHandler);

        return rtn;

    } catch(const Exception::SaguaException& ex) {

        std::printf("%s\n", ex.what());

        exit(1);
    }
}

/// @brief main function
///
/// Process command line arguments. If socket is connected to already executed
/// program, send arguments to that and exit. 
/// Or execute little server thread. After that, run MainManager loop.
int main(int argc, char* argv[])
{
    std::vector<std::string> inputFiles;

    ParseCommandOptions(argc, argv, inputFiles);

    DEBUGTRACE("main");

    std::srand( time(NULL) );
    
    if(!Glib::thread_supported()) {

        DEBUGTRACE_MSG("thread init");

        Glib::thread_init();
    }

    std::auto_ptr<UnixSocketHandler> socketHandler(CreateSocketHandler());

    if(socketHandler->Connect()) {

        socketHandler->Send(inputFiles);
	
        exit(0);
    }

    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);

    SaguaGlobals::mainManager = &CUIMainManager::Instance();

    SaguaGlobals::mainManager->PushList(inputFiles);

    SaguaGlobals::mainManager->Run();

    return 0;
}
