/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file socket.cc
/// @brief Interface class for unix socket source file

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <errno.h>

#include <glibmm.h>

#include "../include/socket.h"
#include "../include/exception.h"

const size_t UnixSocket::integerSize_ = sizeof(int);

/// @brief Constructor of unix socket.
///
/// Prepare path name for socket and set addr to that path.
/// @throw NetException Exception on create socket.
UnixSocket::UnixSocket()
    : fd_(socket(AF_LOCAL, SOCK_STREAM, 0))
{
    if(-1 == fd_)
        throw Exception::NetException("Cannot create socket");

    bzero(reinterpret_cast<char*>(&servAddr_), sizeof(servAddr_));
    GetSocketPath();
    strncpy(servAddr_.sun_path, socketPath_.c_str(), sizeof(servAddr_.sun_path) - 1);

    servAddr_.sun_family = AF_LOCAL;
}

/// @brief Listen to connection.
///
/// If there is file in socket path, unlink it and bind and listen.
/// @return Success of listen
bool UnixSocket::Listen()
{
    if(!UnlinkSocketPath())
        return false;

    if(bind(fd_, reinterpret_cast<struct sockaddr*>(&servAddr_), sizeof(servAddr_)) == -1)
        return false;

    return listen(fd_, 5) != -1;
}

/// @brief Send message to socket.
///
/// Send all size of messages to socket. 
/// @param fd Socket file descriptor
/// @param message Message to send
/// @param size Size of message
/// @return Success of send
bool UnixSocket::Send(const int& fd, const void* message, const int& size)
{
    int sentBytes = -1;
    int currentBytes = 0;
    int leftBytes = size;

    while(currentBytes < size) {
        sentBytes = send(fd, reinterpret_cast<const char*>(message) + currentBytes, leftBytes, 0);
        if(sentBytes == -1)
            return false;

        currentBytes += sentBytes;
        leftBytes -= sentBytes;  
    }

    return true;
}

/// @brief Send message in std::string to socket.
///
/// Send all messages in std::string to socket. 
/// @param fd Socket file descriptor
/// @param message Message to send in std::string
/// @return Success of send
bool UnixSocket::Send(const int& fd, const std::string& message)
{
    const int length = message.size();
    int sentBytes = -1;
    int currentBytes = 0;
    int leftBytes = length;

    while(currentBytes < length) {
        sentBytes = send(fd, message.c_str() + currentBytes, leftBytes, 0);
        if(sentBytes == -1)
            return false;

        currentBytes += sentBytes;
        leftBytes -= sentBytes;  
    }

    return true;
}

/// @brief Send integer 1 to socket
/// @param fd Socket file descriptor
/// @return Success of send
bool UnixSocket::SendSignal(const int& fd)
{
    int buf = 1;

    return Send(fd, &buf, integerSize_);
}

/// @brief Receive from socket and test received data is integer 1
/// @param fd Socket file descriptor
/// @return Success of receive
bool UnixSocket::RecvSignal(const int& fd)
{
    int buf = 0;
    int receivedBytes = recv(fd, &buf, integerSize_, 0);

    return -1 != receivedBytes && 1 == buf;
}

void UnixSocket::GetSocketPath()
{
    socketPath_ = g_get_tmp_dir();
    socketPath_ += "/";
    socketPath_ += PACKAGE_NAME;
    socketPath_ += "-";
    socketPath_ += g_get_user_name();
}

/// @brief If is file in socket path, unlink it.
inline bool UnixSocket::UnlinkSocketPath() const
{
    if(unlink(socketPath_.c_str()) == -1 && errno != ENOENT)
        return false;
    else
        return true;
}
